@@ -1,39 +1,50 @@
+{-# LANGUAGE FlexibleContexts  #-}
 {-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE FlexibleContexts #-}
 -- | 'Enumeratee's to render XML 'Event's. Unlike libxml-enumerator and
 -- expat-enumerator, this module does not provide IO and ST variants, since the
 -- underlying rendering operations are pure functions.
 module Text.XML.Stream.Render
-    ( renderBuilder
+    ( -- * Rendering XML files
+      renderBuilder
     , renderBytes
     , renderText
+    , prettify
+      -- * Renderer settings
     , RenderSettings
     , def
     , rsPretty
     , rsNamespaces
     , rsAttrOrder
+    , rsUseCDATA
     , orderAttrs
-    , prettify
+      -- * Event rendering
+    , tag
+    , content
+      -- * Attribute rendering
+    , Attributes
+    , attr
+    , optionalAttr
     ) where
 
-import Data.XML.Types (Event (..), Content (..), Name (..))
-import Text.XML.Stream.Token
-import qualified Data.Text as T
-import Data.Text (Text)
-import Blaze.ByteString.Builder
-import Data.Conduit.Blaze (builderToByteString)
-import qualified Data.Map as Map
-import Data.Map (Map)
-import Data.Maybe (fromMaybe, mapMaybe)
-import Data.ByteString (ByteString)
-import Data.Default (Default (def))
-import qualified Data.Set as Set
-import Data.List (foldl')
-import Data.Conduit
-import qualified Data.Conduit.List as CL
-import qualified Data.Conduit.Text as CT
-import Data.Monoid (mempty)
-import Control.Monad.Trans.Resource (MonadThrow)
+import           Blaze.ByteString.Builder
+import           Control.Monad.Trans.Resource (MonadThrow)
+import           Data.ByteString              (ByteString)
+import           Data.Conduit
+import           Data.Conduit.Blaze           (builderToByteString)
+import qualified Data.Conduit.List            as CL
+import qualified Data.Conduit.Text            as CT
+import           Data.Default                 (Default (def))
+import           Data.List                    (foldl')
+import           Data.Map                     (Map)
+import qualified Data.Map                     as Map
+import           Data.Maybe                   (fromMaybe, mapMaybe)
+import           Data.Monoid                  (Monoid, mappend, mempty)
+import qualified Data.Set                     as Set
+import           Data.Text                    (Text)
+import qualified Data.Text                    as T
+import           Data.XML.Types               (Content (..), Event (..),
+                                               Name (..))
+import           Text.XML.Stream.Token
 
 -- | Render a stream of 'Event's into a stream of 'ByteString's. This function
 -- wraps around 'renderBuilder' and 'builderToByteString', so it produces
@@ -43,9 +54,9 @@ import Control.Monad.Trans.Resource (MonadThrow)
 --renderBytes :: Monad m => RenderSettings -> Conduit Event m ByteString
 renderBytes rs = renderBuilder rs =$= builderToByteString
 
--- | Render a stream of 'Event's into a stream of 'ByteString's. This function
+-- | Render a stream of 'Event's into a stream of 'Text's. This function
 -- wraps around 'renderBuilder', 'builderToByteString' and 'renderBytes', so it
--- produces optimally sized 'ByteString's with minimal buffer copying.
+-- produces optimally sized 'Text's with minimal buffer copying.
 {-
 renderText :: (MonadThrow m)
            => RenderSettings -> Conduit Event m Text
@@ -53,15 +64,22 @@ renderText :: (MonadThrow m)
 renderText rs = renderBytes rs =$= CT.decode CT.utf8
 
 data RenderSettings = RenderSettings
-    { rsPretty :: Bool
+    { rsPretty     :: Bool
     , rsNamespaces :: [(Text, Text)]
       -- ^ Defines some top level namespace definitions to be used, in the form
       -- of (prefix, namespace). This has absolutely no impact on the meaning
       -- of your documents, but can increase readability by moving commonly
       -- used namespace declarations to the top level.
-    , rsAttrOrder :: Name -> Map.Map Name Text -> [(Name, Text)]
+    , rsAttrOrder  :: Name -> Map.Map Name Text -> [(Name, Text)]
       -- ^ Specify how to turn the unordered attributes used by the "Text.XML"
       -- module into an ordered list.
+    , rsUseCDATA   :: Content -> Bool
+      -- ^ Determines if for a given text content the renderer should use a
+      -- CDATA node.
+      --
+      -- Default: @False@
+      --
+      -- @since 1.3.3
     }
 
 instance Default RenderSettings where
@@ -69,6 +87,7 @@ instance Default RenderSettings where
         { rsPretty = False
         , rsNamespaces = []
         , rsAttrOrder = const Map.toList
+        , rsUseCDATA = const False
         }
 
 -- | Convenience function to create an ordering function suitable for
@@ -96,11 +115,11 @@ orderAttrs orderSpec = order
 -- the blaze-builder package, and allow the create of optimally sized
 -- 'ByteString's with minimal buffer copying.
 renderBuilder :: Monad m => RenderSettings -> Conduit Event m Builder
-renderBuilder RenderSettings { rsPretty = True, rsNamespaces = n } = prettify =$= renderBuilder' n True
-renderBuilder RenderSettings { rsPretty = False, rsNamespaces = n } = renderBuilder' n False
+renderBuilder RenderSettings { rsPretty = True, rsNamespaces = n, rsUseCDATA = useCDATA } = prettify =$= renderBuilder' n True useCDATA
+renderBuilder RenderSettings { rsPretty = False, rsNamespaces = n, rsUseCDATA = useCDATA } = renderBuilder' n False useCDATA
 
-renderBuilder' :: Monad m => [(Text, Text)] -> Bool -> Conduit Event m Builder
-renderBuilder' namespaces0 isPretty = do
+renderBuilder' :: Monad m => [(Text, Text)] -> Bool -> (Content -> Bool) -> Conduit Event m Builder
+renderBuilder' namespaces0 isPretty useCDATA = do
     loop []
   where
     loop nslevels = await >>= maybe (return ()) (go nslevels)
@@ -119,29 +138,34 @@ renderBuilder' namespaces0 isPretty = do
                 yield token
                 loop nslevels'
             _ -> do
-                let (token, nslevels') = eventToToken nslevels e
+                let (token, nslevels') = eventToToken nslevels useCDATA e
                 yield token
                 loop nslevels'
 
-eventToToken :: Stack -> Event -> (Builder, [NSLevel])
-eventToToken s EventBeginDocument =
+eventToToken :: Stack -> (Content -> Bool) -> Event -> (Builder, [NSLevel])
+eventToToken s _ EventBeginDocument =
     (tokenToBuilder $ TokenBeginDocument
             [ ("version", [ContentText "1.0"])
             , ("encoding", [ContentText "UTF-8"])
             ]
      , s)
-eventToToken s EventEndDocument = (mempty, s)
-eventToToken s (EventInstruction i) = (tokenToBuilder $ TokenInstruction i, s)
-eventToToken s (EventBeginDoctype n meid) = (tokenToBuilder $ TokenDoctype n meid [], s)
-eventToToken s EventEndDoctype = (mempty, s)
-eventToToken s (EventCDATA t) = (tokenToBuilder $ TokenCDATA t, s)
-eventToToken s (EventEndElement name) =
+eventToToken s _ EventEndDocument = (mempty, s)
+eventToToken s _ (EventInstruction i) = (tokenToBuilder $ TokenInstruction i, s)
+eventToToken s _ (EventBeginDoctype n meid) = (tokenToBuilder $ TokenDoctype n meid [], s)
+eventToToken s _ EventEndDoctype = (mempty, s)
+eventToToken s _ (EventCDATA t) = (tokenToBuilder $ TokenCDATA t, s)
+eventToToken s _ (EventEndElement name) =
     (tokenToBuilder $ TokenEndElement $ nameToTName sl name, s')
   where
     (sl:s') = s
-eventToToken s (EventContent c) = (tokenToBuilder $ TokenContent c, s)
-eventToToken s (EventComment t) = (tokenToBuilder $ TokenComment t, s)
-eventToToken _ EventBeginElement{} = error "eventToToken on EventBeginElement" -- mkBeginToken False s name attrs
+eventToToken s useCDATA (EventContent c) 
+    | useCDATA c = 
+        case c of
+          ContentText txt -> (tokenToBuilder $ TokenCDATA txt, s)
+          ContentEntity txt -> (tokenToBuilder $ TokenCDATA txt, s)
+    | otherwise  = (tokenToBuilder $ TokenContent c, s)
+eventToToken s _ (EventComment t) = (tokenToBuilder $ TokenComment t, s)
+eventToToken _ _ EventBeginElement{} = error "eventToToken on EventBeginElement" -- mkBeginToken False s name attrs
 
 type Stack = [NSLevel]
 
@@ -326,3 +350,33 @@ nubAttrs orig =
     go (dlist, used) (k, v)
         | k `Set.member` used = (dlist, used)
         | otherwise = (dlist . ((k, v):), Set.insert k used)
+
+
+-- | Generate a complete XML 'Element'.
+tag :: (Monad m) => Name -> Attributes -> Source m Event  -- ^ 'Element''s subnodes.
+                                       -> Source m Event
+tag name (Attributes a) content = do
+  yield $ EventBeginElement name a
+  content
+  yield $ EventEndElement name
+
+-- | Generate a textual 'EventContent'.
+content :: (Monad m) => Text -> Source m Event
+content = yield . EventContent . ContentText
+
+-- | A list of attributes.
+data Attributes = Attributes [(Name, [Content])]
+
+instance Monoid Attributes where
+  mempty = Attributes mempty
+  (Attributes a) `mappend` (Attributes b) = Attributes (a `mappend` b)
+
+-- | Generate a single attribute.
+attr :: Name        -- ^ Attribute's name
+     -> Text        -- ^ Attribute's value
+     -> Attributes
+attr name value = Attributes [(name, [ContentText value])]
+
+-- | Helper function that generates a valid attribute if input isn't 'Nothing', or 'mempty' otherwise.
+optionalAttr :: Name -> Maybe Text -> Attributes
+optionalAttr name = maybe mempty (attr name)
